package de.tu_darmstadt.tk.SmartHomeNetworkSim.core.simpleImplementation;

import java.util.Arrays;
import java.util.Collection;
import java.util.LinkedList;

import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Connection;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Link;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Packet;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Protocol;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.SmartDevice;

public class SimpleConnection implements Connection {

	/**
	 * SmartDevice which is the source of this connection
	 */
	private SmartDevice source;
	/**
	 * SmartDevice which is the destination of this connection
	 */
	private SmartDevice destination;
	/**
	 * Link which connects {@code source} and  {@code destination}
	 */
	private Link link;
	/**
	 * Protocol, which specifies the packet generation
	 */
	private Protocol p;
	
	//for Termination
	/**
	 * SmartDevice which started the termination process
	 */
	private SmartDevice srcOfTermination;
	/**
	 * SmartDevice which responds in the termination process
	 */
	private SmartDevice other;
	
	/**
	 * Creates a new connection between to SmartDevices on a given Link, which communicate with a protocol p
	 * 
	 * @param src SmartDevice which is the source of this connection
	 * @param dest SmartDevice which is the destination of this connection
	 * @param link Link which connects {@code src} and  {@code dest}
	 * @param p Protocol, which specifies the packet generation
	 */
	public SimpleConnection(SmartDevice src, SmartDevice dest, Link link, Protocol p) {
		source = src;
		destination = dest;
		this.link = link;
		this.p = p;
	}
	@Override
	public Link getLink() {
		return link;
	}

	@Override
	public Collection<Packet> simulateTimeInterval(long startTime, long duration) {
		LinkedList<Packet> list = new LinkedList<Packet>();
		for(long i = 0; i<duration; i+=100)
			list.add(p.generateNextPaket(startTime+i));
		return list;
	}
	@Override
	public SmartDevice getSource() {
		return source;
	}
	@Override
	public Collection<SmartDevice> getParticipants() {
		return new LinkedList<SmartDevice>(Arrays.asList(source, destination));
	}
	@Override
	public void removeSmartDevice(SmartDevice sd) {
		//of source == null - connection was already terminated
		if(source == null)return;
		srcOfTermination = sd;
		other = sd == source? destination : source;
		source = null;
		destination = null;	
	}
	
	@Override
	public Collection<Packet> getTerminationPackages(long startTime) {
		return new LinkedList<Packet>(Arrays.asList((Packet)new SimplePacket(startTime, srcOfTermination, other,"Terminated")));
	}

}
