package de.tu_darmstadt.tk.SmartHomeNetworkSim.core.simpleImplementation;

import java.util.Collection;
import java.util.LinkedList;

import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Connection;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Link;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Packet;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.SmartDevice;

import java.util.ArrayList;

/**
 * Simple Implementation of {@link Link}, which allows connection of multiple
 * devices
 *
 * @author Andreas T. Meyer-Berg
 */
public class SimpleLink implements Link {

	/**
	 * Name of the connection
	 */
	private String name;

	/**
	 * Devices connected by this Link
	 */
	private ArrayList<SmartDevice> devices;

	/**
	 * List of packages to store packages sent during simulation intervals, or
	 * after termination
	 */
	private LinkedList<Packet> packets;

	/**
	 * Initializes a simple Link with name and an empty devices list.
	 * 
	 * @param name
	 *            Name the Link should have
	 */
	public SimpleLink(String name) {
		this.name = name;
		this.devices = new ArrayList<SmartDevice>();
		this.packets = new LinkedList<Packet>();
	}

	/**
	 * Returns the name of this Link
	 * 
	 * @return name of this Link
	 */
	public String getName() {
		return name;
	}

	/**
	 * Sets the name of this Link
	 * 
	 * @param name
	 *            the name to set
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * Returns the device that are part of this connection
	 * 
	 * @return the devices of this Link
	 */
	public Collection<SmartDevice> getDevices() {
		return devices;
	}

	/**
	 * Adds the SmartDevice to this Link
	 * 
	 * @param device
	 *            the devices to add
	 */
	public void addDevice(SmartDevice device) {
		this.devices.add(device);
	}

	/**
	 * Remove SmartDevice from this Link
	 * 
	 * @param device
	 *            the device to remove
	 */
	public void removeDevice(SmartDevice device) {
		this.devices.remove(device);
	}

	@Override
	public void simulateTimeInterval(long startTime, long duration) {
		packets.clear();
		for (SmartDevice sd : devices)
			for (Connection c : sd.getConnections()) {
				// Simulate just if source and link match
				if (c.getLink() == this && c.getSource() == sd)
					packets.addAll(c.simulateTimeInterval(startTime, duration));
			}
	}

	@Override
	public Collection<Packet> getPackets() {
		return packets;
	}
}
