package de.tu_darmstadt.tk.SmartHomeNetworkSim.core.simpleImplementation;

import java.util.Arrays;
import java.util.Collection;

import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Packet;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Protocol;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.SmartDevice;

/**
 * Simple Implementation of a protocol, which sends 
 *
 * @author Andreas T. Meyer-Berg
 */
public class SimpleProtocol implements Protocol {

	/**
	 * Source of the protocol, and destination which responds
	 */
	SmartDevice source, destination;
	
	/**
	 * True if the next Package is send by source, false if the next one will be send by destination
	 */
	boolean srcSends;
	
	public SimpleProtocol(SmartDevice src, SmartDevice dest){
		source = src;
		destination = dest;
		srcSends = true;
	}
	@Override
	public Packet generateNextPaket(long timestep) {
		srcSends = !srcSends;
		if(srcSends)
			return new SimplePacket(timestep, destination, source);
		else
			return new SimplePacket(timestep, source, destination);
	}
	
	@Override
	public int getNumberOfRoles() {
		// two different roles
		return 2;
	}
	
	@Override
	public String[] getRoles() {
		return new String[]{"Sender", "Receiver"};
	}
	
	@Override
	public Collection<SmartDevice> getDevicesWithRole(int role) {
		SmartDevice ret = null;
		switch (role) {
		case 0:
			ret = source;
			break;
		case 1:
			ret = destination;
			break;
		default:
			return null;
		}
		if(ret!=null) 
			return Arrays.asList(ret);
		else
			return Arrays.asList();
	}
	
	@Override
	public boolean addDeviceOfRole(SmartDevice device, int role) {
		if(role == 0 && source == null){
			source = device;
		}else if(role == 1 && destination == source){
			destination = device;
		}else{
			return false;
		}
		return false;
	}
	
	@Override
	public void removeDevice(SmartDevice device) {
		if(device == source)
			source = null;
		if(device == destination)
			destination = null;
		
	}
	

}
